<?php

class Extension_Address_Field extends Extension
{
  const TABLE = 'tbl_fields_address';
  
  public function about()
  {
    return array(
      'author' => array(
        'email' => 'jeremy@jeremyboles.com',
        'name' => 'Jeremy Boles',
        'website' => 'http://jeremyboles.com/'
      ),
      'name' => 'Field: Address',
      'release-date' => '2010-05-27',
      'version' => '0.11'
    );
  }
  
  public function appendPreferences($context)
  {
    $fieldset = new XMLElement('fieldset');
    $fieldset->setAttribute('class', 'settings');
    $context['wrapper']->appendChild($fieldset);
    
    $legend = new XMLElement('legend', __('Address Field'));
    $fieldset->appendChild($legend);
    
    $label = Widget::Label(__('Google Maps API Key'));
    $fieldset->appendChild($label);
    
    $key = General::Sanitize($context['parent']->Configuration->get('google-api-key', 'address-field'));
    $input = Widget::Input('settings[address-field][google-api-key]', $key);
    $label->appendChild($input);
    
    $text = __("You'll need to <a href=\"http://code.google.com/apis/maps/signup.html\">grab a Google Maps API key for <em>http://{$_SERVER['HTTP_HOST']}/</em></a> before you can geocode addresses.");
    $p = new XMLElement('p', $text, array('class' => 'help'));
    $fieldset->appendChild($p);
  }
  
  public function getSubscribedDelegates()
  {
    return array(
      array(
        'callback' => 'appendPreferences',
        'delegate' => 'AddCustomPreferenceFieldsets',
        'page' => '/system/preferences/'
      )
    );
  }
  
  public function install()
  {
    $query = "CREATE TABLE `" . self::TABLE . "` (
      `id` int(11) unsigned NOT NULL auto_increment,
      `field_id` int(11) unsigned NOT NULL,
      `geocode` enum('yes','no') NOT NULL default 'yes',
      `hide_country` enum('yes','no') NOT NULL default 'yes',
      `street_label` varchar(80) NOT NULL,
      `city_label` varchar(80) NOT NULL,
      `region_label` varchar(80) NOT NULL,
      `postal_code_label` varchar(80) NOT NULL,
      `country_label` varchar(80) NOT NULL,
      PRIMARY KEY(`id`),
      KEY `field_id` (`field_id`)
    ) TYPE=MyISAM";
    
    return Symphony::Database()->query($query);
  }
  
  public function uninstall()
  {
    $query = "DROP TABLE `" . self::TABLE . "`";
    Symphony::Database()->query($query);
    $this->_Parent->Configuration->remove('google-api-key', 'address-field');
    $this->_Parent->saveConfig();
  }
}