<?php

include_once(TOOLKIT . '/class.gateway.php');

class FieldAddress extends Field
{
  const STYLESHEET = '/extensions/address_field/assets/address-field.css';
  
  public function __construct(&$parent)
  {
    parent::__construct($parent);
    $this->_name = __('Address');
    $this->_required = TRUE;
    $this->set('required', 'no');
  }
  
  public function allowDatasourceOutputGrouping()
  {
    return FALSE;
  }
  
  // Helper method for adding fields to a group on the settings panel
  private function appendGroup(&$wrapper, $fields = array())
  {
    $group = new XMLElement('div');
    $group->setAttribute('class', 'group');
    $wrapper->appendChild($group);
    
    foreach ($fields as $name => $text)
    {
      $label = Widget::Label(__("Label for $text Field"));
      $group->appendChild($label);
      
  	  $value = ($this->get("{$name}_label") ? $this->get("{$name}_label") : $text);
  	  $input = Widget::Input("fields[{$this->get('sortorder')}][{$name}_label]", $value);
  	  $label->appendChild($input);
    }
    
    return $group;
  }
  
  // Helper method for adding a checkbox to a group on the settings panel
  private function appendCheckboxToGroup(&$group, $name, $text)
  {
    $label = Widget::Label();
    $label->setAttribute('class', $name);
    $group->appendChild($label);
    $attributes = array();
    if ($this->get($name) == 'yes')
    {
      $attributes['checked'] = 'true';
    }
    $input = Widget::Input("fields[{$this->get('sortorder')}][{$name}]", 'yes', 'checkbox', $attributes);
    $label->setValue(__("%s $text", array($input->generate())));
  }
  
  public function appendFormattedElement(&$wrapper, $data)
  {
    $address = new XMLElement($this->get('element_name'));
    $wrapper->appendChild($address);
    
    if ($data['longitude'] || $data['latitude'])
    {
      $address->setAttribute('longitude', $data['longitude']);
      $address->setAttribute('latitude', $data['latitude']);
    }
    
    foreach (array('street', 'city', 'region', 'postal_code', 'country') as $name)
    {
      $element = new XMLElement(Lang::createHandle($this->get("{$name}_label")), $data[$name]);
      $element->setAttribute('handle', Lang::createHandle($data[$name]));
      $address->appendChild($element);
    }
  }
  
  // Helper method for added an input to the publish screen
  private function appendInput(&$fieldset, &$data, $field)
  {
    $label = Widget::Label($this->get("{$field}_label"));
    $label->setAttribute('class', $field);
    $fieldset->appendChild($label);
    
    $input = Widget::Input("fields[{$this->get('element_name')}][$field]", $data[$field], 'text');
    $label->appendChild($input);
  }
  
  function buildDSRetrivalSQL($data, &$joins, &$where, $and = FALSE)
  {
    $columns_to_labels = array();
    $where_array = array();
    
    foreach (array('street', 'city', 'region', 'postal_code', 'country') as $name)
    {
      $columns_to_labels[Lang::createHandle($this->get("{$name}_label"))] = $name;
    }
    
    $columns = implode('|', array_keys($columns_to_labels));
    
    $this->_key++;
    $joins .= "
			LEFT JOIN
				`{$this->tableName()}` AS t{$this->get('id')}_{$this->_key}
				ON (e.id = t{$this->get('id')}_{$this->_key}.entry_id)
		";
    
    foreach ($data as $i => $search)
    {
      if (preg_match("/^in ($columns) of (.+)$/", $search, $filters))
      {
        $column = $columns_to_labels[$filters[1]];
        $value = $filters[2];
        
        $where_array[] = "(
						t{$this->get('id')}_{$this->_key}.{$column} = '{$value}'
						OR t{$this->get('id')}_{$this->_key}.{$column}_handle = '{$value}'
					)";
      }
      elseif (preg_match('/^within ([0-9]+)\s?(km|mile|miles) of (.+)$/', $search, $filters))
      {
        $coords = explode(';', $filters[3]);
  			
  			$radius = ($filters[2] == 'km') ? ($filters[1] * 0.621371192) : (float) $filters[1];
  			$earth_radius = 69.1703234283616;
  			
  			$latitude = (float) (1 / $earth_radius) * $radius; 
  			$latMin = $coords[1] - $latitude;
  			$latMax = $coords[1] + $latitude;
  			
  			$longitude = (float) (1 / ($earth_radius * cos($coords[1] * (pi / 180)))) * $radius;
  			$longMin = $coords[0] - $longitude;
  			$longMax = $coords[0] + $longitude;
  			
  			$where_array[] = "(
  			  t{$this->get('id')}_{$this->_key}.`latitude` BETWEEN $latMin AND $latMax
  			  AND t{$this->get('id')}_{$this->_key}.`longitude` BETWEEN $longMin AND $longMax
  			)";
      }
    }
    
    $where .= 'AND (' . implode(($and ? ' AND ' : 'OR'), $where_array) . ')';
    
    return TRUE;
  }
  
  public function canFilter()
  {
    return TRUE;
  }
  
  public function commit()
  {
    if (!parent::commit())
    {
      return FALSE;
    }
    
    $id = $this->get('id');
    if ($id === FALSE)
    {
      return FALSE;
    }
    
    $query = "DELETE FROM `tbl_fields_" . $this->handle() . "` WHERE `field_id` = '$id' LIMIT 1";
    Symphony::Database()->query($query);
    
    $fields = array(
      'field_id' => $id,
      'geocode' => ($this->get('geocode') ? $this->get('geocode') : 'no'),
      'hide_country' => ($this->get('hide_country') ? $this->get('hide_country') : 'no'),
      'street_label' => $this->get('street_label'),
      'city_label' => $this->get('city_label'),
      'region_label' => $this->get('region_label'),
      'postal_code_label' => $this->get('postal_code_label'),
      'country_label' => $this->get('country_label')
    );
    
    return Symphony::Database()->insert($fields, 'tbl_fields_' . $this->handle());
  }
  
  public function createTable()
  {
    $query = "CREATE TABLE IF NOT EXISTS `{$this->tableName()}` (
      `id` int(11) unsigned NOT NULL auto_increment,
      `entry_id` int(11) unsigned NOT NULL,
      `street` varchar(255) NOT NULL,
      `street_handle` varchar(255) NOT NULL,
      `city` varchar(255) NOT NULL,
      `city_handle` varchar(255) NOT NULL,
      `region` varchar(255) NOT NULL,
      `region_handle` varchar(255) NOT NULL,
      `postal_code` varchar(255) NOT NULL,
      `postal_code_handle` varchar(255) NOT NULL,
      `country` varchar(255) NOT NULL,
      `country_handle` varchar(255) NOT NULL,
      `longitude` float(10,7),
      `latitude` float(10,7),
      PRIMARY KEY  (`id`),
      INDEX `entry_id` (`entry_id`),
      INDEX `street` (`street`),
      INDEX `street_handle` (`street_handle`),
      INDEX `city` (`city`),
      INDEX `city_handle` (`city_handle`),
      INDEX `region` (`region`),
      INDEX `region_handle` (`region_handle`),
      INDEX `postal_code` (`postal_code`),
      INDEX `postal_code_handle` (`postal_code_handle`),
      INDEX `country` (`country`),
      INDEX `country_handle` (`country_handle`),
      INDEX `coords` (`longitude`, `latitude`)
    )";
    return Symphony::Database()->query($query);
  }
  
  public function displayPublishPanel(&$wrapper, $data = null, $error = null, $prefix = null, $postfix = null)
  {
    $this->_engine->Page->addStylesheetToHead(URL . self::STYLESHEET);
    
    $fieldset = new XMLElement('fieldset');
    $fieldset->setAttribute('class', 'address-field');
    $wrapper->appendChild($fieldset);
    
    $legend = new XMLElement('legend', $this->get('label'));
    $fieldset->appendChild($legend);
    
    foreach (array('street', 'city') as $field)
    {
      $this->appendInput($fieldset, $data, $field);
    }
    
    $label = Widget::Label($this->get('region_label'));
    $label->setAttribute('class', 'region');
    $fieldset->appendChild($label);
    if ($this->get('hide_country') == 'yes')
    {
      $file = file_get_contents(DOCROOT . '/extensions/address_field/data/us-states.xml');
      $xml = new SimpleXMLElement($file);
      $states = array('');
      foreach($xml as $state)
      {
        $state = (string) $state;
        $value = array($state);
        if ($data['region'] == $state)
        {
          $value[] = TRUE;
        }
        $states[] = $value;
      }
      $region = Widget::Select("fields[{$this->get('element_name')}][region]", $states);
    }
    else
    {
      $region = Widget::Input("fields[{$this->get('element_name')}][region]", $data['region']);
    }
    $label->appendChild($region);
    
    $this->appendInput($fieldset, $data, 'postal_code');
    
    if ($this->get('hide_country') != 'yes')
    {
      $this->appendInput($fieldset, $data, 'country');
    }
  }
  
  public function displaySettingsPanel(&$wrapper, $errors = NULL)
  {
    parent::displaySettingsPanel($wrapper, $errors);
    $this->_engine->Page->addStylesheetToHead(URL . self::STYLESHEET);
    
    $this->appendGroup($wrapper, array('street' => 'Street', 'city' => 'City'));
    $this->appendGroup($wrapper, array('region' => 'Region', 'postal_code' => 'Postal Code'));
    
    $group = $this->appendGroup($wrapper, array('country' => 'Country'));
    $this->appendCheckboxToGroup($group, 'hide_country', 'Hide the country field (US Only)');
    
    $group = $this->appendGroup($wrapper);
    $this->appendCheckboxToGroup($group, 'geocode', 'Geocode this address');
  }
  
  private function getCoords($place)
  {
    $key = $this->_engine->Configuration->get('google-api-key', 'address-field');
    $url = "http://maps.google.com/maps/geo?q=$place&output=xml&key=$key";
    
    $gate = new Gateway;
    $gate->init();
    $gate->setopt('URL', $url, 'address-field');
    $response = $gate->exec();

    // Parse the XML to find the coordinates
    $xml = new SimpleXMLElement($response);
    return preg_split('/,/', $xml->Response->Placemark[0]->Point->coordinates, -1, PREG_SPLIT_NO_EMPTY);
  }
  
  public function processRawFieldData($data, &$status, $simulate = false, $entry_id = NULL)
  {
    if (!is_array($data) || empty($data))
    {
      return NULL;
    }
    
    $country = $data['country'] ? $data['country'] : 'United States of America';
    
    $result = array(
      'street' => $data['street'],
      'city' => $data['city'],
      'region' => $data['region'],
      'postal_code' => $data['postal_code'],
      'country' => $country,
    );
    
    if ($this->get('geocode') == 'yes')
    {
      $place = urlencode(implode(', ', array_values($result)));
      $coords = $this->getCoords($place);
      $result['latitude'] = $coords[1];
      $result['longitude'] = $coords[0];
    }
    
    $result = array_merge($result, array(
      'entry_id' => $entry_id,
      'street_handle' => Lang::createHandle($data['street']),
      'city_handle' => Lang::createHandle($data['city']),
      'region_handle' => Lang::createHandle($data['region']),
      'postal_code_handle' => Lang::createHandle($data['postal_code']),
      'country_handle' => Lang::createHandle($country),
    ));
    
    return $result;
  }
  
  public function isSortable()
  {
    return FALSE;
  }
  
  // Returns the name of the table for storing the individual field's data in
  private function tableName()
  {
    return "tbl_entries_data_{$this->get('id')}";
  }
}